#!/usr/bin/python
#----------------------------------------------------------------------
#
# find_borders.py
#
# Script designed to detect the borders of an scanned image, to crop
# borders from scanned comics and books.
#
# This script loads an image file in memory and tries to detect the
# first top, bottom, left and right lines that has enough pixels
# "active".
#
# A pixel is considered active if (R+G+B)/3 is > 255 - threshold_color.
#
# A scanline is considered "border" if there are N% "active" pixels:
#
# ( if NumPixels_Active_Percent < threshold_percent )
#
# The script returns values for imagemagick's convert "-crop" option:
#
# WIDTHxHEIGHT+xoffset+yoffset
#
# Usage:
#
# $ find_borders.py in.jpg thrshld_color percent_thrshld margins out.jpg
#
# The image out.jpg will detect the borders that have more than 
# "percent_thrshld" pixels active, considering as "active pixels" those
# that (R+G+B/3) < 255-thrshld_color. The resulting size will have 
# "MARGINS" extra pixels around it.
#
# Example:
#
# $ export CROP_SIZE="`find_borders.py in.jpg 20 95 5`"
# $ convert in.jpg -crop $CROP_SIZE out.jpg
#
#----------------------------------------------------------------------

# TODO: Cambiar sistema de parametros a getopt
# TODO: Agregar valores de opciones por defecto
# TODO: Agregar opciones de top_start, bottom_start, left_start, r_start
#       a partir de los que empezar el bucle de deteccion.

import os, sys
import getopt
import pygame
from pygame.locals import *


#----------------------------------------------------------------------
def Usage(argv):
   print "\nUsage:", argv[0], "input.jpg tcolor tpercent margin\n"
   print "input.jpg = Input image"
   print "tcolor = color threshold (255-N) (use 20 as default)"
   print "tpercent = pixels off percentaje (0-100) (use 90 as default)"
   print "margin = border pixels to add to result image (use 5 as default)"
   return 2


#----------------------------------------------------------------------
def main(argv):

   #try:                                
   #    opts, args = getopt.getopt(argv, "pi:c:p:m:", ["input=", "color_tresho="]) 
   #except getopt.GetoptError:           
   #    return Usage(argv)

   # Get arguments from command line
   if len(argv) != 5:
      return Usage(argv)

   name_input = argv[1]

   try:
      tcolor = int(argv[2])
   except:
      print argv[2], "is not a numeric value"
      return 2

   try:
      tpercent = int(argv[3])
   except:
      print argv[3], "is not a numeric value"
      return 2

   try:
      margin = int(argv[4])
   except:
      print argv[4], "is not a numeric value"
      return 2

   # Init pygame:
   if not pygame.font: 
      print 'Warning: pygame.font not available: fonts disabled'

   pygame.init()
   
   image_input, size_input = load_image(name_input, None, 0)
   return Autocrop( image_input, size_input, name_input, tcolor, tpercent, margin )


#----------------------------------------------------------------------
def Autocrop( image_input, size_input, name_input, tcolor, tpercent, margin):

   color_threshold = tcolor
   pixeloff_percent_threshold_h = tpercent
   pixeloff_percent_threshold_v = tpercent
   marginx, marginy = margin, margin
   image_width = size_input[2]
   image_height = size_input[3]

   # Find top margin
   done, y = 0, 0
   while ( done == 0 and y < image_height ):
      howmany, percent = Count_Horizontal( image_input, size_input, y, color_threshold )
      if percent < pixeloff_percent_threshold_h:
         done = 1
      y += 1
   top_margin = y

   # Find bottom margin
   done, y = 0, image_height-1
   while ( done == 0 and y > 0 ):
      howmany, percent = Count_Horizontal( image_input, size_input, y, color_threshold )
      if percent < pixeloff_percent_threshold_h:
         done = 1
      y -= 1
   bottom_margin = y

   # Find left margin
   done, x = 0, 0
   while ( done == 0 and x < image_width ):
      howmany, percent = Count_Vertical( image_input, size_input, x, color_threshold )
      if percent < pixeloff_percent_threshold_v:
         done = 1
      x += 1
   left_margin = x

   # Find right margin
   done, x = 0, image_width-1
   while ( done == 0 and x > 0 ):
      howmany, percent = Count_Vertical( image_input, size_input, x, color_threshold )
      if percent < pixeloff_percent_threshold_v:
         done = 1
      x -= 1
   right_margin = x

   width = right_margin-left_margin+(2*marginx)
   height = bottom_margin-top_margin+(2*marginy)
   offsetx = left_margin-marginx
   offsety = top_margin-marginy

   # Check that new image is not larger than original:
   if width >= image_width:
      width = image_width
      offsetx = 0
   if height >= image_height:
      height = image_height
      offsety = 0

   print "%sx%s+%s+%s" % ( width, height, offsetx, offsety ) 
   return 1


#----------------------------------------------------------------------
def Count_Horizontal( surface, size, y, threshold ):

   total = 0

   # Count number of pixels active ( != 255-threshold)
   for x in range(0, size[2]):
      gray = sum( list(surface.get_at( (x, y) )[0:3]) )/ 3
      if gray > 255 - threshold:
         total += 1

   return total, (float(total) / float(size[2])) * 100.0


#----------------------------------------------------------------------
def Count_Vertical( surface, size, x, threshold ):

   total = 0
   # Count number of pixels active ( != 255-threshold)
   for y in range(0, size[3]):
      gray = sum( list(surface.get_at( (x, y) )[0:3]) )/ 3
      if gray > 255 - threshold:
         total += 1

   return total, (float(total) / float(size[3])) * 100.0


#----------------------------------------------------------------------
def load_image(name, colorkey=None, doconvert=1 ):
    try:
        image = pygame.image.load(name)
    except pygame.error, message:
        print 'Cannot load image:', name
        raise SystemExit, message
    if doconvert == 1:
       image = image.convert()
       if colorkey is not None:
           if colorkey is -1:
               colorkey = image.get_at((0,0))
           image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()


#----------------------------------------------------------------------
def save_image(name, surface):
   try:
      pygame.image.save(surface, name)
   except pygame.error, message:
      print 'Cannot load image:', name
      raise SystemExit, message
   return


#----------------------------------------------------------------------
if __name__ == "__main__":
   sys.exit(main(sys.argv))

